(function(){
  $(function(){    
    $("input[type=number][min='0']").change(function(){
      var num = parseFloat($(this).val());
      if(num < 0){
        $(this).val(num * -1);
      }
    });
    
    $("a.help").popover({trigger: 'hover'});
    $("#roiParams").find("input, select").on('keyup change', function(){
      myComputeRoi();
    });
    myComputeRoi();
  });

var roiData = {};

function myComputeRoi() {
  roiData.currency = $('#currencySelect').val();
  roiData.dev_count = parseFloat($('#devCount').val());
  roiData.dev_hourly_wage = parseFloat($('#hourlyWage').val());
  roiData.redeploy_time = parseFloat($('#redeployTime').val());
  roiData.redeploys_per_hour = parseFloat($('#redeploysPerHour').val());

  var jrebelYearlyCost = LS.jrebelPrices[roiData.currency];

  var roi = computeRoi(
    roiData.redeploy_time * 60,
    roiData.redeploys_per_hour,
    roiData.dev_hourly_wage,
    LS.developerEfficiency,
    jrebelYearlyCost
  );
  
  function fixedNumOrNothing(num, fixedTo){
    if(isNaN(num) || num === Number.NEGATIVE_INFINITY || num === Number.POSITIVE_INFINITY) {
      return "";
    }else{
      return num.toFixed(fixedTo || 0);
    }
  }

  var minutesSavedPerDayPerDevel = LS.work_hours_per_day * roi['timeSavedPerHour'] / 60;
  var moneySavedPerYear = LS.work_hours_per_year * roi['moneySavedPerHour'] * roiData.dev_count;
  var weeksSavedPerYear = LS.work_hours_per_year * roi['timeSavedPerHour'] * roiData.dev_count / (3600 * 40);
  var payoffTimeInDays = roi['payoffTime'] / (3600 * LS.work_hours_per_day);
  
  $('#minutesSavedPerDayPerDevel').html(fixedNumOrNothing(minutesSavedPerDayPerDevel));
  $('#weeksSavedPerYear').html(fixedNumOrNothing(weeksSavedPerYear, 1));
  $('#moneySavedPerYear').html(fixedNumOrNothing(moneySavedPerYear).replace(/\B(?=(\d{3})+(?!\d))/g, ","));
  $('#currency').html(roiData.currency);
  $('#payoffTimeInDays').html(fixedNumOrNothing(payoffTimeInDays, 1));

  return false;
}

/*
 * (float) redeployTime -- time of redeploy, in seconds
 * (float) redeploysPerHour -- number of redeploys done per hour
 * (float) hourlyWage -- hourly wage of one developer
 * (float) efficiency -- developer efficiency
 * (float) jrebelYearlyCost -- JRebel Enterprise yearly license price
 *
 * Note: all times are in seconds if not stated otherwise (*Hour, *Year, etc.).
 */
function computeRoi(redeployTime, redeploysPerHour, hourlyWage, efficiency, jrebelYearlyCost) {
  var timeSavedPerHour = redeployTime * redeploysPerHour;
  var efficientTimeSavedPerHour = timeSavedPerHour * efficiency;
  var efficientMoneySavedPerHour = hourlyWage * (efficientTimeSavedPerHour / 3600);

  var roi = [];
  roi['timeSavedPerHour'] = efficientTimeSavedPerHour;
  roi['moneySavedPerHour'] = (timeSavedPerHour / 3600) * hourlyWage;
  roi['payoffTime'] = jrebelYearlyCost / (efficientMoneySavedPerHour / 3600);

  return roi;
}

})();
