@echo off

echo.

if "%OS%" == "Windows_NT" setlocal

if "%OS%" == "Windows_NT" (
  set "DIRNAME=%~dp0%"
) else (
  set DIRNAME=.\
)

echo OS is: %OS%
echo.

pushd %DIRNAME%..
if "x%LS_HOME%" == "x" (
  set "LS_HOME=%CD%"
)
popd

echo Detected LS_HOME: %LS_HOME%
cd %LS_HOME%

if "x%JAVA_HOME%" == "x" (
  set JAVA=java
  echo JAVA_HOME is not set. Unexpected results may occur.
  echo Set JAVA_HOME to the directory of your local JDK to avoid this message.
) else (
  set JAVA="%JAVA_HOME%\bin\java"
)

echo Detected JAVA_HOME: %JAVA_HOME%

echo.

if ""%1"" == ""run"" goto doRun
if ""%1"" == ""start"" goto doStart
if ""%1"" == ""stop"" goto doStop
if ""%1"" == ""status"" goto doStatus
if ""%1"" == ""restart"" goto doRestart

echo Usage:  license-server command ( options ... )
echo commands:
echo   run               Start License Server in the current window
echo   start             Start License Server in a separate window
echo   restart           Restart License Server
echo   stop              Stop License Server
echo   status            Check License Server status
goto end


:: ============================ doRun ============================ 

:doRun
  shift
  goto execCmd


:: ============================ doStart ============================ 

:doStart
  shift
  if not "%OS%" == "Windows_NT" goto noTitle
  if "%TITLE%" == "" set TITLE=License Server
  set JAVA=start "%TITLE%" %JAVA%
  set START=1
  goto gotTitle


:noTitle
  set JAVA=start %JAVA%


:gotTitle
  goto execCmd


:: ============================ doRestart ============================ 

:doRestart
  echo Preparing for restarting LS2 ..
  set RESTART=1
  goto doStop


:: ============================ doStop ============================ 

:doStop
  echo Preparing for stopping LS2 ..
  shift
  set STOP=1
  goto execCmd


:: ============================ doStatus ============================ 

:doStatus
  echo Preparing for status query ..
  shift
  set STATUS=1
  goto execCmd


:: ============================ execCmd ============================ 
:: Handle the arguments, execute the appropriate commend

:execCmd
  set CMD_LINE_ARGS=

  :: --- Handle the arguments
  :: For each argument, append it to CMD_LINE_ARGS variable.
  :: If no more arguments to take, stop.

  :setArgs
    if ""%1""=="""" (
      goto doneSetArgs
    )
    set CMD_LINE_ARGS=%CMD_LINE_ARGS% %1
    shift
    goto setArgs
    :doneSetArgs
    echo Done parsing command line arguments

:: --- make the final decision which command we should really invoke

:: =========== A) STOP the server, exit the script (except for if it is followed by a restart)
if "%STOP%" == "1" (
  
  echo Stopping the server ..

  echo Executing command: %JAVA% -classpath "%LS_HOME%\lib\license-server.jar" %JAVA_OPTS% %CMD_LINE_ARGS% com.zeroturnaround.ls.Shutdown 
  %JAVA% -classpath "%LS_HOME%\lib\license-server.jar" %JAVA_OPTS% %CMD_LINE_ARGS% com.zeroturnaround.ls.Shutdown
  
  if "%RESTART%" == "1" (
    echo Will restart now ..
    set RESTART=
    set STOP=
    goto doStartImpl
  )
  goto end
)

:: =========== B) make a STATUS query
if "%STATUS%" == "1" (
  echo.
  echo Executing command: %JAVA% -classpath "%LS_HOME%\lib\license-server.jar" %JAVA_OPTS% %CMD_LINE_ARGS% com.zeroturnaround.ls.Status
  %JAVA% -classpath "%LS_HOME%\lib\license-server.jar" %JAVA_OPTS% %CMD_LINE_ARGS% com.zeroturnaround.ls.Status
  goto end
)

:: =========== C) START the server

:doStartImpl

  :: ----- before starting the server, check if we need to clean up old PID file of the dead process (happens after ungraceful kill, OS crash, ...)
  :doPidCheck
    echo Checking for pidfile ..  

    :: path of the PID-file, if exists
    set PIDFILE=%~dp0\RUNNING_PID
    echo Expecting pidfile at path %PIDFILE%
  
    if not exist "%PIDFILE%" (
      echo Pidfile doesn't exist
      goto endOfPidCheck
    )

    echo Pidfile exist.. reading its contents
    :: read the PID from pidfile
    set /p MYPID=<"%PIDFILE%"
    echo Old RUNNING_PID is %MYPID%

    :: check if such a process is alive (ask for tasklist, grep for the PID, see if it can be found)
    echo Checking if the process is alive ..
    tasklist /FI "PID eq %MYPID%" 2>NUL | find /I /N "%MYPID%">NUL

    if not "%ERRORLEVEL%"=="0" (
      echo Old License Server process looks dead.. deleting old pidfile
      del "%PIDFILE%"
    )

    :endOfPidCheck
    echo Pidfile checking is complete


  :: Start the server
  echo.
  echo Executing command: %JAVA% -classpath "%LS_HOME%\lib\license-server.jar" %JAVA_OPTS% %CMD_LINE_ARGS% com.zeroturnaround.ls.Startup
  %JAVA% -classpath "%LS_HOME%\lib\license-server.jar" %JAVA_OPTS% %CMD_LINE_ARGS% com.zeroturnaround.ls.Startup

  if "%START" == "1" (
    echo Starting License Server.
    goto end
  )


:end