(function () {
  var charts = [];
  
  /**
   * Create a chart
   */
  var createChart = function(type){
    
    var chart = {
      plot: null,
      from: LS.chart.seriesConf.from,
      fromTXT: "",
      to: LS.chart.seriesConf.to,
      toTXT: "",
      interval: LS.chart.seriesConf.interval,
      type: type,
      url: LS.chart.dataUrl,
      csvUrl: LS.chart.csvUrl,
      seriesData: LS.chart.seriesData[type],
      totalTokenId: LS.chart.seriesConf.totalTokenId,
      
      /**
       * Initialization method. Called on chart creation
       */
      init: function(){
        var that = this;
        var seriesOpts = {
          points: {
            show: false
          },
          lines: {
            show: true
          },
          legend: {
            show: false
          },
          tooltip: true,
          tooltipOpts: {
            content: function(point){
              if(that.type.toLowerCase().match("timesaved$")=='timesaved'){
                return new Date(point[0]).format("mmm dd. HH:MM") + " - " + jintervals(point[1]/1000, "{Hours? and }{minutes}");
              }else{
                return new Date(point[0]).format("mmm dd. HH:MM") + " - " + point[1];
              }
            }
          },
          grid: {
            hoverable: true
          },
          xaxis: {
            mode: 'time',
            timezone: "browser",
            minTickSize: [1, 'minute']
          },
          yaxis: {
            min: 0,
            minTickSize: 1
          }
        };
        
        if(that.type.toLowerCase().match("timesaved$") == 'timesaved'){
          $.extend(seriesOpts.yaxis, {
            tickFormatter: this.formatDuration,
            mode: 'time',
            minTickSize: [1, 'minute']
          });
        }else{
          seriesOpts.yaxis.tickDecimals = 0;
        }
  
        // Draw the chart..?
        var data = that.normalize(that.seriesData);
        // associate it with either 'chart-timeSaved' or 'chart-seatsInUse' element.
        that.plot = $.plot($("#chart-" + type), [data], seriesOpts);
        that.assignColors();
        
        //Very very strange bug hack fix. Axis labels don't appear right away =(
        setTimeout(function(){
          that.plot.setData(that.plot.getData());
          that.plot.setupGrid();
          that.plot.draw();
        }, 500);
      },
      formatDuration: function(val) {
        if (val == 0)
          return "";
        var d = parseInt(val / (1000 * 60 * 60 * 24 * 365));
        if (d != 0)
          return d + "y" + chart.formatDuration(val - d *(1000 * 60 * 60 * 24 * 365) );
        d = parseInt(val / (1000 * 60 * 60 * 24 * 30));
        if (d != 0)
          return d + "mo" + chart.formatDuration(val - d *(1000 * 60 * 60 * 24 * 30) );
        d = parseInt(val / (1000 * 60 * 60 * 24 * 7));
        if (d != 0)
          return d + "w" + chart.formatDuration(val - d *(1000 * 60 * 60 * 24 * 7) );
        d = parseInt(val / (1000 * 60 * 60 * 24));
        if (d != 0)
          return d + "d" + chart.formatDuration(val - d *(1000 * 60 * 60 * 24) );
        d = parseInt(val / (1000 * 60 * 60));
        if (d != 0)
          return d + "h" + chart.formatDuration(val - d *(1000 * 60 * 60) );
        d = parseInt(val / (1000 * 60));
        if (d != 0)
          return d + "m" + chart.formatDuration(val - d *(1000 * 60) );
        d = parseInt(val / (1000));
        if (d != 0)
          return d + "s" + chart.formatDuration(val - d *(1000) );
        if (val != 0)
          return d + "ms";
    
        return "";
      },
      normalize: function(seriesData) {
        var tick = this.getTruncatedTick();
        var c = 0;
        while (tick <= new Date(this.to)) {
          if (seriesData.data[c] && seriesData.data[c][0] <= tick.getTime()) {
          }
          else {
            seriesData.data.splice(c, 0, [tick.getTime(), 0]);
          }
          c++;
          tick = this.getNextTick(tick);
        }
        
        if(seriesData.data.length == 1){
          seriesData.points = {show: true};
        }
        return seriesData;
      },
      getTruncatedTick: function() {
        var tick = new Date(this.from);
        if (this.interval == 12)
          return new Date(tick.getFullYear(), tick.getMonth(), tick.getDate(), tick.getHours(), tick.getMinutes(), 0, 0);
        else if (this.interval == 11) // HOUR_OF_DAY
          return new Date(tick.getFullYear(), tick.getMonth(), tick.getDate(), tick.getHours(), 0, 0, 0);
        else if (this.interval == 5) // DAY_OF_MONTH
          return new Date(tick.getFullYear(), tick.getMonth(), tick.getDate(), tick.getHours(), 0, 0, 0);
        else if (this.interval == 2) // MONTH
          return new Date(tick.getFullYear(), tick.getMonth(), 1, 0, 0, 0, 0);
        else if (this.interval == 1) // YEAR
          return new Date(tick.getFullYear(), 0, 1, 0, 0, 0, 0);
        return tick;
      },
      getNextTick: function(date) {
        if (this.interval == 12) // MINUTE
          date.setMinutes(date.getMinutes() + 1);
        else if (this.interval == 11) // HOUR_OF_DAY
          date.setHours(date.getHours() + 1);
        else if (this.interval == 5) // DAY_OF_MONTH
          date.setDate(date.getDate() + 1);
        else if (this.interval == 2) // MONTH
          date.setMonth(date.getMonth() + 1);
        else if (this.interval == 1) // YEAR
          date.setFullYear(date.getFullYear() + 1);
        return date;
      },
      fullUrl: function(id){
        return this.url + '?' + $.param({
          id: id,
          interval: this.interval,
          from: this.fromTXT,
          to: this.toTXT
        });
      },
      fullCsvUrl: function(){
        return this.csvUrl + '?' + $.param({
          ids: _.map(chart.plot.getData(), function(serie){
            return serie.label;
          }).join(","),
          interval: this.interval,
          from: this.fromTXT,
          to: this.toTXT
        });
      },
      // Reset the charts object (draw it again)
      reset: function() {
        var that = this;
        $.getJSON(this.fullUrl(this.totalTokenId), function (response) {
          if(that.interval == 2){
            that.plot.getAxes().xaxis.options.minTickSize = [1, 'month'];
          }else if(that.interval == 1){
            that.plot.getAxes().xaxis.options.minTickSize = [1, 'year'];
          }else{
            that.plot.getAxes().xaxis.options.minTickSize = [1, 'minute'];
          }
          that.plot.setData([that.normalize(response)]);
          that.plot.resize();
          that.plot.setupGrid();
          that.plot.draw();
          that.assignColors();
        });
      },
      assignColors: function() {
        var series = this.plot.getData();
        $("#legend li .legend-color").css("background", "lightgrey");
        for (var i = 0; i < series.length; ++i)
          $("#token-" + series[i].label + " .legend-color").css("background", series[i].color);
      },
      downloadCsv: function(){
        document.location = this.fullCsvUrl();
      },
      toggleTokenChart: function(id){
        var data = chart.plot.getData();
        for (var i = 0; i < data.length; i++) {
          if (data[i].label == id) {
            data.splice(i, 1);
            chart.plot.setData(data);
            chart.plot.resize();
            chart.plot.setupGrid();
            chart.plot.draw();
            chart.assignColors();
            return;
          }
        }
  
        $.getJSON(chart.fullUrl(id), function (response) {
          var data = chart.plot.getData();
          data.push(chart.normalize(response));
          chart.plot.setData(data);
          chart.plot.resize();
          chart.plot.setupGrid();
          chart.plot.draw();
          chart.assignColors();
        });
      }
    };
    charts.push(chart);
    return chart;
  };

  // HTML attribute id chart-timeSaved or chart-seatsInUse. They are for example present on the dashboard.
  // Does this code replace those placeholders..?
  if(LS.chart.types.length == 1 && $("#chart").length > 0) {
    $("#chart").attr("id", "chart-" + LS.chart.types[0]);
  }
  
  // For each chart-type, create the chart
  _.each(LS.chart.types, function(type){
    createChart(type).init();
  });
  
  $(function () {

    $("#legend li").click(function () {
      var id = $(this).data("id");
      charts[0].toggleTokenChart(id);
    });

    $('#interval').change(function () {
      charts[0].interval = $(this).val();
      charts[0].reset();
    });
    
    $('#date-range-field').click(function () {
      $(this).toggleClass("opened");
      var calendar = $('#datepicker-calendar');
      calendar.toggle();
      if (calendar.is(":hidden")) {
        var datePair = calendar.DatePickerGetDate()[0];
        charts[0].from = datePair[0].getTime();
        charts[0].fromTXT  = datePair[0].getFullYear()+"-"+(1+datePair[0].getMonth())+"-"+datePair[0].getDate();
        charts[0].to = datePair[1].getTime();
        charts[0].toTXT = datePair[1].getFullYear()+"-"+(1+datePair[1].getMonth())+"-"+datePair[1].getDate();
        console.log("From:"+charts[0].fromTXT+" To:"+charts[0].toTXT);
        charts[0].reset();
      }
    });

    var to = new Date(charts[0].to);
    var from = new Date(charts[0].from);
    $('#datepicker-calendar').DatePicker({
      inline: true,
      date: [from, to],
      calendars: 3,
      mode: 'range',
      current: new Date(to.getFullYear(), to.getMonth() - 1, 1),
      onChange: function (dates, el) {
        // update the range display
        $('#date-range-field span').text(
          dates[0].getDate() + ' ' + dates[0].getMonthName(true) + ', ' +
            dates[0].getFullYear() + ' - ' +
            dates[1].getDate() + ' ' + dates[1].getMonthName(true) + ', ' +
            dates[1].getFullYear());
      }
    });
    
    $("#downloadCsv").click(function(){
      charts[0].downloadCsv();
    });
  });
})();


